package netaddr_test

import (
	. "idiolink/go/netaddr"
	"testing"
)

func TestIPParsesCIDR(t *testing.T) {
	actual, _ := IP("192.0.2.0/24")
	expected :=
		&NetAddr{Address: []byte{192, 0, 2, 0}, Length: byte(24)}

	if !IsEqual(actual, expected) {
		t.Errorf("Expected %+v but got %+v", expected, actual)
	}
}

func TestIPParsesIPv4AddressWithContext(t *testing.T) {
	actual, _ := IP("192.0.2.1/24")
	expected :=
		&NetAddr{Address: []byte{192, 0, 2, 1}, Length: byte(24)}

	if !IsEqual(actual, expected) {
		t.Errorf("Expected %+v but got %+v", expected, actual)
	}
}

func TestIPParsesIPv6AddressWithContext(t *testing.T) {
	actual, _ := IP("2001:db8::1/64")
	expected :=
		&NetAddr{
			Address: []byte{
				0x20, 0x01, 0x0d, 0xb8,
				0x00, 0x00, 0x00, 0x00,
				0x00, 0x00, 0x00, 0x00,
				0x00, 0x00, 0x00, 0x01,
			},
			Length: byte(64),
		}

	if !IsEqual(actual, expected) {
		t.Errorf("Expected %+v but got %+v", expected, actual)
	}
}

func TestIPFailsToParseJunk(t *testing.T) {
	_, err := IP("256.0.0.0")
	if err == nil {
		t.Error("Expected error but got nil")
	}
}

func TestIPFailsToParseOtherJunk(t *testing.T) {
	_, err := IP("255.0.0.0.0")
	if err == nil {
		t.Error("Expected error but got nil")
	}
}

func TestContainsIsNotStrict(t *testing.T) {
	na, _ := IP("192.0.2.0/24")
	actual := Contains(na, na)
	expected := true

	if actual != expected {
		t.Errorf("Expected %+v but got %+v", expected, actual)
	}
}

func TestContainsReturnsTrue(t *testing.T) {
	na1, _ := IP("192.0.2.0/24")
	na2, _ := IP("192.0.2.0/25")
	actual := Contains(na1, na2)
	expected := true

	if actual != expected {
		t.Errorf("Expected %+v but got %+v", expected, actual)
	}
}

func TestContainsReturnsFalseOnShorterLen(t *testing.T) {
	na1, _ := IP("192.0.2.0/24")
	na2, _ := IP("192.0.2.0/25")
	actual := Contains(na2, na1)
	expected := false

	if actual != expected {
		t.Errorf("Expected %+v but got %+v", expected, actual)
	}
}

func TestContainsReturnsFalseOnAdjacent(t *testing.T) {
	na1, _ := IP("192.0.2.0/25")
	na2, _ := IP("192.0.2.128/25")
	actual := Contains(na1, na2)
	expected := false

	if actual != expected {
		t.Errorf("Expected %+v but got %+v", expected, actual)
	}
}

func TestLastAddressWorks(t *testing.T) {
	na, _ := IP("192.0.2.96/27")
	actual := na.LastAddress()
	expected :=
		&NetAddr{
			Address: []byte{192, 0, 2, 127},
			Length:  byte(27),
		}

	if !IsEqual(actual, expected) {
		t.Errorf("Expected %+v but got %+v", expected, actual)
	}
}
